use serde::{Deserialize, Serialize};

#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct Message {
    pub from: String,
    pub message_kind: MessageKind,
    pub content: String,
    // placeholder for the future
    // pub previous: String,
}

#[derive(Debug, Clone, Serialize, Deserialize)]
pub enum MessageKind {
    Test,
    Move,
    CreatePlayer,
    CreateLobby,
}

impl Message {
    pub fn new(from: &str, message_kind: MessageKind, content: impl Into<String>) -> anyhow::Result<Self> {
        let from = from.into();
        let content = content.into();

        Ok(Self {
            from,
            message_kind,
            content,
        })
    }

    pub fn encode(&self) -> anyhow::Result<String> {
        let message_json: String = serde_json::to_string(self)?;

        Ok(message_json)
    }

    pub fn decode(buffer: &[u8]) -> anyhow::Result<Self> {
        let message: Message = serde_json::from_slice(buffer)?;

        Ok(message)
    }

    pub fn as_bytes_vec(&self) -> anyhow::Result<Vec<u8>> {
        let message_json = self.encode()?;
        let message_bytes: &[u8] = message_json.as_bytes();
        let bytes_length: u32 = message_bytes.len() as u32;

        let mut buffer = Vec::new();
        buffer.extend(bytes_length.to_be_bytes());
        buffer.extend(message_bytes);

        Ok(buffer)
    }
}
